(*  BSL for MSP430F149 with use of a RAM our own RAM BSL
    The ROM BSL is booted and a mass erase is done
    Then an empty password is sent
    Now it loads a RAM BSL to address 220.
    It starts the RAM BSL by jumping to address 220
    Now the target is programmed, this takes about 28 seconds.
    Finally the target is reset. The data is stored in XRAM!

  Connections from programmer to target
    Programmer         Target 
    --------------------------------
    P3.0 = Output   -> P2.2 RX
    P3.1 = Input    -> P1.1 TX
    P6.0 = Test     -> TCK
    P6.1 = Reset    -> RST
    VCC             -> VCC
    GND             -> GND
    --------------------------------
*)

hex
inside also definitions
code BL-EMIT    ( char -- ) \ RS232s Char to RS232
    100 # tos bis           \ Add stopbit
    tos tos add             \ Add startbit
    0A # moon mov           \ 1 + 8 + Stop bits
    begin,
        tos rrc             \ Get next bit to carry
        cs? if,
            #1 222 & bis    \ P3OUT Send one
        else,
            #1 222 & bic    \ P3OUT Send zero
        then,
        wait-bit # call     \ Wait bittime
        #1 moon sub         \ Send all bits
    =? until,
    sp )+ tos mov
    next
end-code

: RESET-TO-BSL2 ( -- )
    cr ." Use TCK to start build-in BSL "
    01 243 *bis  2 243 *bic  10 ms  \ TCK high, Reset low & wait
    01 243 *bic  1 ms  1 243 *bis  1 ms \ TCK pulse 1
    01 243 *bic  1 ms  02 243 *bis  \ TCK pulse 2 & BSL reset
    1 ms  01 243 *bis  1 ms ;       \ Wait and release TCK

: MASS-ERASE    ( -- )              \ Erase whole flash
    cr ." Do mass erase "  dm 20 0 do
        FF00 A56A 00 18 command  waitack
    loop ;

: LOAD-PC       ( a -- )            \ Start program at address 'a'
    cr ." Start custom BSL "
    0000 00 1A command nack? ?abort ; \ Load PC

\ Output one data word in uppercase hexadecimal ASCII
: ONE-WORD      ( x -- )
    0  3 0 do  10 um/mod  0  loop   \ Convert to 4-digits
    drop  4 0 do
         >dig bl-emit               \ Send these digits
    loop ;

\ Write a binary part from XRAM using the BSL
: UPLOAD-XPART  ( ea1 +n -- ea2 )
    0 ?do x@+ one-word  loop ; \ -- ea1        Write one word at a time

\ Write binary from XRAM stored at dir entry '+n' using own BSL
: UPLOAD-XRAM   ( +n -- )
     cr ." Write binary from XRAM "
    'entries 2x@  2dup d.       \ -- xa         File start address
     begin
        x@+ ?dup while          \ -- xa1 adr    Fetch dest. address
        ch A bl-emit one-word   \ -- xa1        Send address command
        x@+  ch W bl-emit       \ -- xa2        Start Write command
        2/ dup one-word         \ -- xa3        Send length in words
        upload-xpart            \ -- xa3        One continuous part at a time
     repeat
    2drop ;

forth definitions
: PROGRAM-F     ( '+n' -- )     \ Program the binary from address '+n'
    setup-bsl  reset-to-bsl2    \ to the MSP430Fxxxx series
    mass-erase                  \ Delete whole Flash to FF
    empty-pass  send-password
    0 xwrite-flash              \ Write own RAM BSL to addr. 220
    0220 load-pc A0 ms          \ Boot to new RAM BSL
    upload-xram  5 ms           \ Write flash from XRAM
    reset-target ;              \ Wait and restart target

fresh  shield F149\  freeze

\ End
